package de.jottyfan.minecraft.quickiefabric.blocks;

import java.util.Arrays;
import java.util.List;
import java.util.Set;

import com.mojang.serialization.MapCodec;

import de.jottyfan.minecraft.quickiefabric.items.QuickieItems;
import net.fabricmc.fabric.api.object.builder.v1.block.FabricBlockSettings;
import net.minecraft.block.AbstractBlock;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.Blocks;
import net.minecraft.block.CropBlock;
import net.minecraft.block.ShapeContext;
import net.minecraft.entity.Entity;
import net.minecraft.entity.ai.pathing.NavigationType;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.item.ItemConvertible;
import net.minecraft.item.ItemStack;
import net.minecraft.loot.context.LootContextParameterSet.Builder;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.ActionResult;
import net.minecraft.util.Hand;
import net.minecraft.util.ItemScatterer;
import net.minecraft.util.collection.DefaultedList;
import net.minecraft.util.hit.BlockHitResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.random.Random;
import net.minecraft.util.shape.VoxelShape;
import net.minecraft.world.BlockView;
import net.minecraft.world.World;
import net.minecraft.world.WorldView;

/**
 *
 * @author jotty
 *
 */
public class BlockSulforpad extends CropBlock {

	public BlockSulforpad(AbstractBlock.Settings settings) {
		super(settings.nonOpaque().blockVision(Blocks::never).solidBlock(Blocks::never));
		setDefaultState(getDefaultState().with(this.getAgeProperty(), 0));
	}

	protected BlockSulforpad() {
		super(FabricBlockSettings.create());
	}

	public static final Set<Block> getAllowedBlockHolder(){
		return Set.of(Blocks.LAVA, Blocks.LAVA_CAULDRON, Blocks.MAGMA_BLOCK);
	}

	@Override
	public void randomTick(BlockState state, ServerWorld world, BlockPos pos, Random random) {
		growNow(state, world, pos, random);
	}

	private void generateNeighbor(ServerWorld world, BlockPos pos, Random random) {
		int xOffset = random.nextInt(3) - 1;
		int zOffset = random.nextInt(3) - 1;
		BlockPos newPos = new BlockPos(pos).add(xOffset, 0, zOffset);
		if (getAllowedBlockHolder().contains(world.getBlockState(newPos).getBlock()) && world.getBlockState(newPos.up()).isAir()) {
			world.setBlockState(newPos, QuickieBlocks.SULFORPAD.getDefaultState(), Block.NOTIFY_LISTENERS);
		}
	}

	private void growNow(BlockState state, ServerWorld world, BlockPos pos, Random random) {
		int i;
		if ((i = this.getAge(state)) < this.getMaxAge()) {
			world.setBlockState(pos, this.withAge(i + 1), Block.NOTIFY_LISTENERS);
		} else {
			generateNeighbor(world, pos, random);
		}
	}

	@Override
	public void grow(ServerWorld world, Random random, BlockPos pos, BlockState state) {
		growNow(state, world, pos, random);
	}

	@Override
	public boolean canPathfindThrough(BlockState state, BlockView world, BlockPos pos, NavigationType type) {
		return true;
	}

	@Override
	public List<ItemStack> getDroppedStacks(BlockState state, Builder builder) {
		Integer age = this.getAge(state);
		Integer number = age < 3 ? 1 : (age < 6 ? 2 : 3);
		return Arrays.asList(new ItemStack[] { new ItemStack(QuickieItems.SULPHOR, number) });
	}

	@Override
	public VoxelShape getOutlineShape(BlockState state, BlockView world, BlockPos pos, ShapeContext context) {
		return Block.createCuboidShape(1.0, 0.0, 1.0, 15.0, 1.5, 15.0);
	}

	@Override
	protected boolean canPlantOnTop(BlockState floor, BlockView world, BlockPos pos) {
		return getAllowedBlockHolder().contains(floor.getBlock()) && world.getBlockState(pos.up()).isAir();
	}

	@Override
	protected ItemConvertible getSeedsItem() {
		return QuickieItems.SULPHOR;
	}

	private void spawnHarvested(World world, BlockPos pos, BlockState state) {
		DefaultedList<ItemStack> list = DefaultedList.of();
		getDroppedStacks(state, null).forEach(itemStack -> {
			list.add(itemStack);
		});
		ItemScatterer.spawn(world, pos, list);
	}

	@Override
	public ActionResult onUse(BlockState state, World world, BlockPos pos, PlayerEntity player, Hand hand,
			BlockHitResult hitResult) {
		if (!world.isClient && isMature(state)) {
			spawnHarvested(world, pos, state);
			Random random = world.getRandom();
			world.setBlockState(pos, state.with(AGE, random.nextInt(6)));
		}
		return ActionResult.PASS;
	}

	@Override
	public void onEntityCollision(BlockState state, World world, BlockPos pos, Entity entity) {
		// do nothing here
	}

	@Override
	public ItemStack getPickStack(WorldView world, BlockPos pos, BlockState state) {
		return new ItemStack(QuickieItems.SULPHOR, 1);
	}

	@Override
	public MapCodec<? extends BlockSulforpad> getCodec() {
		return BlockSulforpad.createCodec(BlockSulforpad::new);
	}
}
