package de.jottyfan.minecraft.quickiefabric.blocks;

import java.util.HashMap;
import java.util.Map;

import de.jottyfan.minecraft.quickiefabric.items.QuickieItems;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.FallingBlock;
import net.minecraft.entity.ItemEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.Items;
import net.minecraft.state.property.IntProperty;
import net.minecraft.text.Text;
import net.minecraft.util.ActionResult;
import net.minecraft.util.Hand;
import net.minecraft.util.hit.BlockHitResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

/**
 *
 * @author jotty
 *
 */
public abstract class DrillBlock extends FallingBlock {
	private static final Integer MAX_FUEL = 255;
	public static final IntProperty FUEL = IntProperty.of("fuel", 0, MAX_FUEL);

	public DrillBlock(Settings settings) {
		super(settings);
		setDefaultState(getDefaultState().with(FUEL, 0));
	}

	@Override
	public BlockState onBreak(World world, BlockPos pos, BlockState state, PlayerEntity player) {
		Integer fuelLeft = state.get(FUEL);
		world.spawnEntity(new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(), new ItemStack(QuickieItems.CANOLABOTTLE, fuelLeft)));
		return super.onBreak(world, pos, state, player);
	}

	@Override
	protected void appendProperties(net.minecraft.state.StateManager.Builder<Block, BlockState> builder) {
		builder.add(FUEL);
	}

	@Override
	public ActionResult onUse(BlockState state, World world, BlockPos pos, PlayerEntity player, Hand hand,
			BlockHitResult hit) {
		Map<Item, Integer> loadings = new HashMap<>();
		loadings.put(QuickieItems.CANOLABOTTLE, 8);
		loadings.put(QuickieItems.CANOLABOTTLESTACK, 72);
		ItemStack stack = player.getStackInHand(hand);
		Item item = stack.getItem();
		if (stack.isEmpty() || !loadings.containsKey(item) ) {
			if (world.isClient()) {
				player.sendMessage(Text.translatable("msg.drill.fuelinfo", state.get(FUEL)));
			}
		} else {
			Integer fuelWeight = loadings.get(item);
			if (fuelWeight != null) {
				Integer load = MAX_FUEL - state.get(FUEL);
				if (load < fuelWeight) {
					Integer numberOfTooMuchLoad = fuelWeight - load;
					fuelWeight = load;
				}
				world.setBlockState(pos, state.with(FUEL, state.get(FUEL) + fuelWeight));
				if (item.equals(QuickieItems.CANOLABOTTLE)) {
					world.spawnEntity(new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(), new ItemStack(Items.GLASS_BOTTLE, 1)));
				} else if (item.equals(QuickieItems.CANOLABOTTLESTACK)) {
					world.spawnEntity(new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(), new ItemStack(Items.GLASS_BOTTLE, 9)));
				}
				stack.decrement(1);
			}
		}
		return ActionResult.PASS;
	}
}
