package de.jottyfan.minecraft.quickiefabric.tools;

import java.util.List;

import com.google.common.collect.Lists;

import net.fabricmc.fabric.api.item.v1.FabricItemSettings;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.item.ItemStack;
import net.minecraft.item.ItemUsageContext;
import net.minecraft.item.ShovelItem;
import net.minecraft.item.ToolMaterials;
import net.minecraft.util.ActionResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.world.World;

/**
 *
 * @author jotty
 *
 */
public class ToolQuickiepowderShovel extends ShovelItem implements ToolRangeable {
	public static final Integer DEFAULT_HARVEST_RANGE = 6;
	public HarvestRange range;

	public ToolQuickiepowderShovel() {
		super(ToolMaterials.DIAMOND, 5, 2.0f, new FabricItemSettings());
		this.range = new HarvestRange(DEFAULT_HARVEST_RANGE);
	}

	private void createPathOnGrass(World world, BlockPos pos, Direction side) {
		BlockState blockState = world.getBlockState(pos);
		if (blockState.isAir()) {
			// try to find one underneath
			pos = pos.down();
			blockState = world.getBlockState(pos);
		} else if (!world.getBlockState(pos.up()).isAir()) {
			pos = pos.up();
			blockState = world.getBlockState(pos);
		}
		if (side != Direction.DOWN) {
			BlockState blockState2 = (BlockState) PATH_STATES.get(blockState.getBlock());
			if (blockState2 != null && world.getBlockState(pos.up()).isAir()) {
				if (!world.isClient) {
					world.setBlockState(pos, blockState2, 11);
				}
			}
		}
	}

	@Override
	public ActionResult useOnBlock(ItemUsageContext context) {
		World world = context.getWorld();
		BlockPos pos = context.getBlockPos();
		BlockPos[] positions = new BlockPos[] { pos.north().north().west().west(), pos.north().north().west(),
				pos.north().north(), pos.north().north().east(), pos.north().north().east().east(), pos.north().west().west(),
				pos.north().west(), pos.north(), pos.north().east(), pos.north().east().east(), pos.west().west(), pos.west(),
				pos, pos.east(), pos.east().east(), pos.south().west().west(), pos.south().west(), pos.south(),
				pos.south().east(), pos.south().east().east(), pos.south().south().west().west(), pos.south().south().west(),
				pos.south().south(), pos.south().south().east(), pos.south().south().east().east() };
		for (BlockPos p : positions) {
			createPathOnGrass(world, p, context.getSide());
		}
		return super.useOnBlock(context);
	}

	@Override
	public HarvestRange getRange(ItemStack stack) {
		// TODO: get range from stack
		return range;
	}

	@Override
	public boolean canBreakNeighbors(BlockState blockState) {
		return super.isSuitableFor(blockState);
	}

	@Override
	public List<Block> getBlockList(Block block) {
		return Lists.newArrayList(block);
	}

//	@Override
//	public ActionResult<ItemStack> onItemRightClick(World worldIn, PlayerEntity playerIn, Hand handIn) {
//		CommonToolCode.onItemRightClick(worldIn, playerIn, handIn);
//		return super.onItemRightClick(worldIn, playerIn, handIn);
//	}
//
//	@Override
//	public void addInformation(ItemStack stack, World worldIn, List<ITextComponent> tooltip, ITooltipFlag flagIn) {
//		CommonToolCode.addInformation(stack, worldIn, tooltip, flagIn);
//		super.addInformation(stack, worldIn, tooltip, flagIn);
//	}
}
